<?php
namespace Teknisa\SM\Controller;

use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00001100LancamentoPropostaService;
use Teknisa\SM\Service\Sm00001100ItemLancamentoPropostaService;

class Sm00001100ItemLancamentoPropostaController extends SMCrud{
    protected $dataSourceName       = 'fct_itemorcamentoalimenticio_widget';
    protected $dataSourceNameToSave = 'fct_itemorcamento';
    protected $itensService;
    protected $environment;
    protected $propostaService;
    protected $newCode;

    public function __construct ($crudService, $environment, Sm00001100ItemLancamentoPropostaService $itensService, Sm00001100LancamentoPropostaService $propostaService, NewCode $newCode){
        parent::__construct($crudService, $environment);
        $this->itensService    = $itensService;
        $this->environment     = $environment;
        $this->propostaService = $propostaService;
        $this->newCode         = $newCode;
    }
    
    /**
     * Salva inserções e edições dos itens do orcamento
    **/
    public function save(Request $request, Response $response){
        try{
            $nrorg = $this->environment->getNrOrg();
            $row   = $request->getDataSet()->getRows()[0];
            if (!$row["__is_new"]){
                parent::save($request, $response);
            } else {
                $nritemorcamento = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
                $row["NRITEMORCAMENTO"]  = $nritemorcamento;
                parent::save($request, $response);
                $columnsToReturn = $this->crudService->getColumnsToReturn();
                $columnsToReturn[0]["NRITEMORCAMENTO"] = $nritemorcamento;
                $columnsToReturn[0]["NRORG"]           = $nrorg;
                $response->addDataSet(new DataSet("columnsToReturn", $columnsToReturn));
            }
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda atualizar os valores de cada elementos do custo alimentar de acordo
     * com o desconto ou acrescimo dado
    **/
    public function replicarAlteracoesItensFilhos(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $cdservorca     = $row["CDSERVORCA"];
            $vrdiferenca    = $row["VRDIFERENCA"];
            $nrorg          = $this->environment->getNrOrg();
            $this->itensService->replicarAlteracoesItensFilhos($nrseqorcamento, $cdservorca, $vrdiferenca, $nrorg);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza as quantidades dos elementos na tabela de serviço
    **/
    public function salvarNovasQuantidadesElemento(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $this->itensService->salvarNovasQuantidadesElemento($row, $cdoperador, $nrorg);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Deleta um item do orcamento
    **/
    public function deletarItemOrcamento(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->itensService->excluiItemAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg);
            if($row["IDTIPO"] == "M"){
                $this->itensService->inativarElementosProposta($nrseqorcamento, $nrorg, $row["CDSERVORCA"], $row["IDARVORE"], $row["NRTIPOORCAMENTO"], $cdoperador);
                $this->itensService->alterarValorItemPai($nrseqorcamento, explode("_", $row["IDARVOREPAI"])[0], $nrorg, $cdoperador, explode("_", $row["IDARVOREPAI"])[1], $row["VRTOTALITEMORCA"]);
            }
            
            $request->setParameter("dbname", $this->dataSourceNameToSave);
            parent::delete($request, $response);
            $this->propostaService->atualizarValorProposta($nrseqorcamento, $nrorg);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Insere na tabela auxilar as formaçãoes de preço utilizadas na proposta
     * (sempre irá inserir nela para garantir que o orcamento utilize apenas itens selecionados)
    **/
    public function insereMaoObraFormacao(Request\Row $request, Response $response){
        try{
            $row            = $request->getRow();
            $formacoes      = $row["FORMPRECOSERVICOS"];
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrorg          = $this->environment->getNrOrg();
            $cdoperador     = $this->environment->getCdOperador();
            $nritemorcamento = -1;
            
            foreach($formacoes as $formacao) {
                $this->itensService->executarProcedureFctCopiaFormPreco($formacao["NRFORMPRECOSERVICO"], $nrorg, $cdoperador, $nrseqorcamento, $nritemorcamento, $formacao["NRCALCFORMPRECSERV"], $formacao["IDFORMACAO"]);    
                //$formacao["IDFORMACAO"] =  $formacao["IDFORMACAO"] + 1;
            }
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza a quantidade da formação de preço na proposta e recalcula o seu valor
    **/
    public function atualizarQuantidadeMObraItens(Request $request, Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            foreach ($rows as $row) {
                $nrseqorcamento = $row["NRSEQORCAMENTO"];
                if($row["__is_new"])
                    $nrrelacformacao = $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true);
                else
                    $nrrelacformacao = $row["NRRELACFORMACAOPROP"];
                
                $formacao = array(
                    "__is_new"            => $row["__is_new"],
                    "NRRELACFORMACAOPROP" => $nrrelacformacao,
                    "NRFORMPRECOSERVICO"  => $row["NRFORMPRECOSERVICO"],
                    "NRSEQORCAMENTO"      => $row["NRSEQORCAMENTO"],
                    "NRORG"               => $nrorg,
                    "NRORGULTATU"         => $nrorg,
                    "CDOPERULTATU"        => $cdoperador,
                    "QTFORMACAOPROP"      => $row["QTFORMACAOPROP"],
                    "CDSERVORCARELAC"     => $row["CDSERVORCA"]
                );
                $requestRelac = new Request\DataSet(new DataSet("", [$formacao]), "product#/fct_relacformacaoprop/save", "POST", $request->getUserId());
                $this->dataSourceNameToSave = "fct_relacformacaoprop";
                $requestRelac->setParameter('dbname', 'fct_relacformacaoprop');
                parent::save($requestRelac, $response);
                
                if($row["__is_new"])
                    $this->itensService->executarProcedureFctCopiaFormPreco($row["NRFORMPRECOSERVICO"], $nrorg, $cdoperador, $nrseqorcamento, 0, $row["NRCALCFORMPRECSERV"], $nrrelacformacao);
            }
            $this->dataSourceNameToSave = "fct_orcamento";
            $this->propostaService->atualizarValorItem($nrseqorcamento, $nrorg, $cdoperador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza a quantidade da formação de preço na proposta e recalcula o seu valor
    **/
    public function deletarMaoObraItens(Request $request, Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            foreach ($rows as $row) {
                $nrseqorcamento = $row["NRSEQORCAMENTO"];
                $formacao[] = array(
                    "NRRELACFORMACAOPROP" => $row["NRRELACFORMACAOPROP"],
                    "NRSEQORCAMENTO"      => $row["NRSEQORCAMENTO"],
                    "NRORG"               => $nrorg
                );
                $requestRelac = new Request\DataSet(new DataSet("", $formacao), "product#/fct_relacformacaoprop/remove", "POST", $request->getUserId());
                $requestRelac->setParameter('dbname', 'fct_relacformacaoprop');
                parent::delete($requestRelac, $response);
                $this->itensService->deletarItensFormacaoProp($row["NRFORMPRECOSERVICO"], $nrseqorcamento, $nrorg, $row["NRRELACFORMACAOPROP"]);
            }
            $this->dataSourceNameToSave = "fct_orcamento";
            $this->propostaService->atualizarValorItem($nrseqorcamento, $nrorg, $cdoperador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * deleta um elemento ou uma sub-árvore de elementos 
     * da mão de obra ligada a proposta
    **/
    public function deletarElementosFormacaoProp(Request\Row $request, Response $response){
        try{
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nrrelacformacao = $row["NRRELACFORMACAOPROP"];
            $nrformacao      = $row["NRFORMPRECOSERVICO"];
            $nrcalcformacao  = $row["NRCALCFORMPRECSERV"];
            $this->itensService->deletarElementosFormacaoProp($nrformacao, $nrseqorcamento, $nrorg, $nrrelacformacao, $nrcalcformacao);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Volta os itens de formação de preço para os valores de antes da edição
    **/
    public function voltarBackupItensMaoObra(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $nrrelacformacao = $row["NRRELACFORMACAOPROP"];
            $nrformacao      = $row["NRFORMPRECOSERVICO"];
            $nrcalcformacao  = $row["NRCALCFORMPRECSERV"];
            $idformacao      = $row["IDFORMACAO"];
            $this->itensService->deletarItensFormacaoProp($nrformacao, 0, $nrorg, $nrrelacformacao);
            $this->itensService->executarProcedureFctCopiaFormPreco($nrformacao, $nrorg, $cdoperador, 0, -1, $nrcalcformacao, $idformacao);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}